FUNCTION_BLOCK Lcd_WriteString
VAR_INPUT

    I2cHandle_i                 : HI2C;
    usiLine_i                   : USINT;
    strText_i                   : STRING(40);
    xEnable_i                   : BOOL;

END_VAR

VAR_OUTPUT

    usiError_o                  : USINT             := 0;
    xConfirm_o                  : BOOL              := FALSE;

END_VAR

VAR

    FB_Lcd_Write_Cmd            : Lcd_Write_Cmd;

    iTextLen                    : INT;
    iCharIdx                    : INT;
    strChar                     : STRING;
    usiChrCcode                 : USINT;

    bCommand                    : BYTE;
    bMode                       : BYTE;

    xConfirm                    : BOOL;
    usiError                    : USINT;

    FB_RTrig                    : R_TRIG;
    iProcState                  : INT               := 0;

END_VAR


    FB_RTrig (CLK := xEnable_i);
    IF (FB_RTrig.Q = TRUE) THEN
        iTextLen   := LEN (strText_i);
        iCharIdx   := 1;
        iProcState := 0;
        usiError_o := 0;
        xConfirm_o := FALSE;
    END_IF;

    IF (xEnable_i = FALSE) THEN
        RETURN;
    END_IF;


    CASE iProcState OF

    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.lcd_display_string.lcd_write({LINE_REG})                                  *)
    (* ------------------------------------------------------------------------------------ *)

    0:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    1:
        CASE usiLine_i OF
        1:      bCommand := 16#80;
        2:      bCommand := 16#C0;
        3:      bCommand := 16#94;
        4:      bCommand := 16#D4;
        ELSE    usiError := 128;
        END_CASE;

        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle_i,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.lcd_display_string.lcd_write({ASC | LCD_RS})                              *)
    (* ------------------------------------------------------------------------------------ *)

    2:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    3:
        strChar := MID (strText_i, 1, iCharIdx);
        usiChrCcode := ASC (strChar);
        bCommand := USINT_TO_BYTE (usiChrCcode);
        bMode    := LCD_RS;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle_i,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            IF (iCharIdx < iTextLen) THEN
                iCharIdx := iCharIdx + 1;
                iProcState := iProcState - 1;
            ELSE
                iProcState := iProcState + 1;
			END_IF;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  Finish                                                                              *)
    (* ------------------------------------------------------------------------------------ *)

    4:  (* ---- Finish ---- *)

        usiError_o := 0;
        xConfirm_o := TRUE;
        RETURN;


    (* ------------------------------------------------------------------------------------ *)
    (*  Error                                                                               *)
    (* ------------------------------------------------------------------------------------ *)

    -1:  (* ---- Error ---- *)
        usiError_o := usiError;
        xConfirm_o := FALSE;
        RETURN;

    ELSE
        RETURN;

    END_CASE;



    RETURN;




END_FUNCTION_BLOCK